/*
**  Simplified resource database module.
**  
**  Two databases are maintained.  The active and the defaults database.  The
**  active database contains the resource values from which current behaviour
**  is derived.  The defaults database contains the program defaults.
**  
**  The program specifies default values by either explicity placing them into
**  the defaults database, or by specifying them on fetches from the active
**  database.
**  
**  The action of using defaults involves copying the defaults database
**  resource values to the active database.
**  
**  The action of saving current settings involves writing the active database
**  to a disk file.  Restoring saved settings is done by reading the disk file
**  into the active database.
**  
**  Behavioural changes requested by the user at run time are stored in the
**  active database.  When the program needs to obtain a behavioural value
**  from the database, it calls these routines.
*/

#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */
#include "xtsu_database.h"

private XrmDatabase active   = NULL;
private XrmDatabase defaults = NULL;
private XrmDatabase current  = NULL;

public void xtsu_db_set_database ( XrmDatabase display ) { active = display; }

/*
**  Return the file specification of the active database disk file
*/
public char *xtsu_db_filename ( )
{
    static char name[256];
    char *dir;
        
#ifdef VMS
    dir = "decw$user_defaults:";
    strcpy ( name, dir );
    strcat ( name, "xts.dat" );
#else
    dir = getenv ( "XAPPLRESDIR" );
    if ( dir == NULL ) dir = ".";
    strcpy ( name, dir );
    strcat ( name, "/" );
    strcat ( name, "Xts" );
#endif
    return ( name );
}

/*
**  Put a string resource value into the defaults database
*/
public void xtsu_db_put_default_string
(
    char *resource,
    char *data
)
{
    XrmValue value;
    value.addr = data;
    value.size = strlen(data)+1;
    XrmPutResource ( &defaults, resource, XtRString, &value );
}

/*
**  Put an integer resource value into the defaults database
*/
public void xtsu_db_put_default_int
(
    char *resource,
    int data
)
{
    char text[32];
    XrmValue value;
    sprintf ( text, "%d", data );
    value.addr = &text;
    value.size = strlen(text)+1;
    XrmPutResource ( &defaults, resource, XtRString, &value );
}

/*
**  Set current behaviour to default values
*/
public void xtsu_db_use_default
(
)
{
    XrmMergeDatabases ( defaults, &current );
    XrmMergeDatabases ( current, &active );
}

public void xtsu_db_use_current
(
)
{
    XrmMergeDatabases ( current, &active );
}

/*
**  Place a string value into the current database
*/
public void xtsu_db_put_string
(
    char *resource,
    char *data
)
{
    XrmValue value;
    value.addr = data;
    value.size = strlen(data)+1;
    XrmPutResource ( &current, resource, XtRString, &value );
    XrmPutResource ( &active, resource, XtRString, &value );
}

/*
**  Place a integer value into the current database
*/
public void xtsu_db_put_int
(
    char *resource,
    int data
)
{
    char text[32];
    XrmValue value;
    sprintf ( text, "%d", data );
    value.addr = &text;
    value.size = strlen(text)+1;
    XrmPutResource ( &current, resource, XtRString, &value );
    XrmPutResource ( &active, resource, XtRString, &value );
}

/*
**  Obtain a string value from the current database
*/
public char *xtsu_db_get_string
(
    char *resource,
    char *defaults
)
{
    XrmValue value;
    XrmString type;

    xtsu_db_put_default_string ( resource, defaults );
    if ( current != NULL )
    {
        /* look in current */
        XrmGetResource ( current, resource, "", &type, &value );
        if ( type != NULL ) 
        {
            /* resource found in current */
            /* if it is a string type, return a pointer to it */
            if ( XrmStringsEqual ( type, XtRString ) ) return value.addr;
        }
    }

    /* resource not found in current */
    /* look in active */
    XrmGetResource ( active, resource, "", &type, &value );
    if ( type != NULL )
    {
        /* resource found in active */
        /* copy to current */
        XrmPutResource ( &current, resource, type, &value );
        /* if it is a string type, return a pointer to it */
        if ( XrmStringsEqual ( type, XtRString ) ) return value.addr;
    }
    else
    {
        /* resource not found in active */
        /* apply default to current and active */
        xtsu_db_put_string ( resource, defaults );
        /* return default value */
        return defaults;
    }
    return "";
}

/*
**  Obtain an integer value from the current database
*/
public int xtsu_db_get_int
(
    char *resource,
    int defaults
)
{
    XrmValue value = { 0, 0 };
    XrmString type = 0;

    xtsu_db_put_default_int ( resource, defaults );
    if ( current != NULL )
    {
        XrmGetResource ( current, resource, "", &type, &value );
        if ( type != NULL )
        {
            /* resource found in current */
            if ( XrmStringsEqual ( type, XtRInt ) ) return *((int *)value.addr);
            if ( XrmStringsEqual ( type, XtRString ) ) return atoi(value.addr);
        }
    }


    /* resource not found in current */
    /* look in active */
    XrmGetResource ( active, resource, "", &type, &value );
    if ( type != NULL )
    {
        /* resource found in active */
        /* copy to current */
        XrmPutResource ( &current, resource, type, &value );
        if ( XrmStringsEqual ( type, XtRInt ) ) return *((int *)value.addr);
        if ( XrmStringsEqual ( type, XtRString ) ) return atoi(value.addr);
    }
    else
    {
        /* resource not found in active */
        /* apply default to current and active */
        xtsu_db_put_int ( resource, defaults );
        /* return default value */
        return defaults;
    }
    return 0;
}

/*
**  Write or read the current database to or from the current database disk file
*/
public void xtsu_db_save_current ()
{
    XrmPutFileDatabase ( current, xtsu_db_filename() );
}

public void xtsu_db_read_current ()
{
    XrmDatabase file = NULL;
    file = XrmGetFileDatabase ( xtsu_db_filename() );
    XrmMergeDatabases ( file, &current );
    XrmDestroyDatabase ( file );
}
